@@ -29,9 +29,13 @@ class MiscFuncs
 {
     public:
         MiscFuncs() { }
-        ~MiscFuncs() { };
+        ~MiscFuncs() { }
         string asString(int n);
         string asString(long long n);
+#ifdef __x86_64__
+        //remove ambiguity while compiling for x64 arch
+        string asString(size_t n);
+#endif
         string asString(long n);
         string asString(unsigned int n, unsigned int width = 0);
         string asString(unsigned char c) { return asString((unsigned int)c); }
@@ -39,8 +43,9 @@ class MiscFuncs
         string asLongString(float n);
         string asHexString(int x);
         string asHexString(unsigned int x);
-        float string2float(string str);
-        int string2int(string str);
+        static float string2float(string str);
+        static int string2int(string str);
+        static unsigned int string2uint(string str);
         bool isRegFile(string chkpath);
         bool isDirectory(string chkpath);
         bool isFifo(string chkpath);
@@ -49,6 +54,14 @@ class MiscFuncs
         float rap2dB(float rap);
 };
 
+void invSignal(float *sig, size_t len);
+
+template<class T>
+T limit(T val, T min, T max)
+{
+    return val < min ? min : (val > max ? max : val);
+}
+
 inline float MiscFuncs::dB2rap(float dB) { return exp10f((dB) / 20.0f); }
 inline float MiscFuncs::rap2dB(float rap) { return 20.0f * log10f(rap); }
 
