@@ -24,18 +24,18 @@
 
 #include "Params/SUBnoteParameters.h"
 
-SUBnoteParameters::SUBnoteParameters() : Presets()
+SUBnoteParameters::SUBnoteParameters(SynthEngine *_synth) : Presets(_synth)
 {
     setpresettype("SUBnoteParameters");
-    AmpEnvelope = new EnvelopeParams(64, 1);
+    AmpEnvelope = new EnvelopeParams(64, 1, synth);
     AmpEnvelope->ADSRinit_dB(0, 40, 127, 25);
-    FreqEnvelope = new EnvelopeParams(64, 0);
+    FreqEnvelope = new EnvelopeParams(64, 0, synth);
     FreqEnvelope->ASRinit(30, 50, 64, 60);
-    BandWidthEnvelope = new EnvelopeParams(64, 0);
+    BandWidthEnvelope = new EnvelopeParams(64, 0, synth);
     BandWidthEnvelope->ASRinit_bw(100, 70, 64, 60);
 
-    GlobalFilter = new FilterParams(2, 80, 40);
-    GlobalFilterEnvelope = new EnvelopeParams(0, 1);
+    GlobalFilter = new FilterParams(2, 80, 40, synth);
+    GlobalFilterEnvelope = new EnvelopeParams(0, 1, synth);
     GlobalFilterEnvelope->ADSRinit_filter(64, 40, 64, 70, 60, 64);
     defaults();
 }
