@@ -36,7 +36,7 @@
 #include "Synth/SUBnote.h"
 
 SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
-                 float velocity, int portamento_, int midinote, bool besilent) :
+                 float velocity, int portamento_, int midinote, bool besilent, SynthEngine *_synth) :
     pars(parameters),
     GlobalFilterL(NULL),
     GlobalFilterR(NULL),
@@ -46,7 +46,9 @@ SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
     log_0_01(logf(0.01f)),
     log_0_001(logf(0.001f)),
     log_0_0001(logf(0.0001f)),
-    log_0_00001(logf(0.00001f))
+    log_0_00001(logf(0.00001f)),
+    synth(_synth),
+    filterStep(0)
 {
     ready = 0;
 
@@ -55,7 +57,7 @@ SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
 
     // Initialise some legato-specific vars
     Legato.msg = LM_Norm;
-    Legato.fade.length = lrintf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
+    Legato.fade.length = (int)truncf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
     if (Legato.fade.length < 1)
         Legato.fade.length = 1;// (if something's fishy)
     Legato.fade.step = (1.0f / Legato.fade.length);
@@ -490,11 +492,16 @@ inline void SubFilterB(const float coeff[4], float &src, float work[4])
 //in quite a bit of wasted time
 void SUBnote::filter(bpfilter &filter, float *smps)
 {
+    if(synth->getIsLV2Plugin()){
+        filterVarRun(filter, smps);
+        return;
+    }
+
     assert(synth->buffersize % 8 == 0);
     float coeff[4] = {filter.b0, filter.b2,  -filter.a1, -filter.a2};
     float work[4]  = {filter.xn1, filter.xn2, filter.yn1, filter.yn2};
 
-    for(int i = 0; i < synth->buffersize; i += 8) {
+    for(int i = 0; i < synth->p_buffersize; i += 8) {
         SubFilterA(coeff, smps[i + 0], work);
         SubFilterB(coeff, smps[i + 1], work);
         SubFilterA(coeff, smps[i + 2], work);
@@ -510,26 +517,66 @@ void SUBnote::filter(bpfilter &filter, float *smps)
     filter.yn2 = work[3];
 }
 
+//Andrew Deryabin: support for variable-length runs
+//currently only for lv2 plugin
+void SUBnote::filterVarRun(SUBnote::bpfilter &filter, float *smps)
+{
+    float tmpout;
+    int runLength = synth->p_buffersize;
+    int i = 0;
+    if(runLength >= 8){
+        float coeff[4] = {filter.b0, filter.b2,  -filter.a1, -filter.a2};
+        float work[4]  = {filter.xn1, filter.xn2, filter.yn1, filter.yn2};
+        while(runLength >= 8){
+            SubFilterA(coeff, smps[i + 0], work);
+            SubFilterB(coeff, smps[i + 1], work);
+            SubFilterA(coeff, smps[i + 2], work);
+            SubFilterB(coeff, smps[i + 3], work);
+            SubFilterA(coeff, smps[i + 4], work);
+            SubFilterB(coeff, smps[i + 5], work);
+            SubFilterA(coeff, smps[i + 6], work);
+            SubFilterB(coeff, smps[i + 7], work);
+            i += 8;
+            runLength -= 8;
+        }
+        filter.xn1 = work[0];
+        filter.xn2 = work[1];
+        filter.yn1 = work[2];
+        filter.yn2 = work[3];
+    }
+
+    for(; i < synth->p_buffersize; ++i){
+        tmpout=smps[i] * filter.b0 + filter.b2 * filter.xn2
+               -filter.a1 * filter.yn1 - filter.a2 * filter.yn2;
+        filter.xn2=filter.xn1;
+        filter.xn1=smps[i];
+        filter.yn2=filter.yn1;
+        filter.yn1=tmpout;
+        smps[i]=tmpout;
+    }
+
+}
+
 
 // Init Parameters
 void SUBnote::initparameters(float freq)
 {
-    AmpEnvelope = new Envelope(pars->AmpEnvelope, freq);
+    AmpEnvelope = new Envelope(pars->AmpEnvelope, freq, synth);
     if (pars->PFreqEnvelopeEnabled != 0)
-        FreqEnvelope = new Envelope(pars->FreqEnvelope, freq);
+        FreqEnvelope = new Envelope(pars->FreqEnvelope, freq, synth);
     else
         FreqEnvelope = NULL;
     if (pars->PBandWidthEnvelopeEnabled != 0)
-        BandWidthEnvelope = new Envelope(pars->BandWidthEnvelope, freq);
+        BandWidthEnvelope = new Envelope(pars->BandWidthEnvelope, freq, synth);
     else
         BandWidthEnvelope = NULL;
     if (pars->PGlobalFilterEnabled != 0)
     {
         globalfiltercenterq = pars->GlobalFilter->getq();
-        GlobalFilterL = new Filter(pars->GlobalFilter);
+        GlobalFilterL = new Filter(pars->GlobalFilter, synth);
         if (stereo != 0)
-            GlobalFilterR = new Filter(pars->GlobalFilter);
-        GlobalFilterEnvelope = new Envelope(pars->GlobalFilterEnvelope, freq);
+            GlobalFilterR = new Filter(pars->GlobalFilter, synth);
+        GlobalFilterEnvelope = new Envelope(pars->GlobalFilterEnvelope, freq, synth);
         GlobalFilterFreqTracking = pars->GlobalFilter->getfreqtracking(basefreq);
     }
     computecurrentparameters();
@@ -639,21 +686,21 @@ void SUBnote::computecurrentparameters(void)
 // Note Output
 int SUBnote::noteout(float *outl, float *outr)
 {
-    memset(outl, 0, synth->bufferbytes);
-    memset(outr, 0, synth->bufferbytes);
+    memset(outl, 0, synth->p_bufferbytes);
+    memset(outr, 0, synth->p_bufferbytes);
     if (!NoteEnabled)
         return 0;
 
     // left channel
-    for (int i = 0; i < synth->buffersize; ++i)
+    for (int i = 0; i < synth->p_buffersize; ++i)
         tmprnd[i] = synth->numRandom() * 2.0f - 1.0f;
     for (int n = 0; n < numharmonics; ++n)
     {
         float rolloff = overtone_rolloff[n];
-        memcpy(tmpsmp, tmprnd, synth->bufferbytes);
+        memcpy(tmpsmp, tmprnd, synth->p_bufferbytes);
         for (int nph = 0; nph < numstages; ++nph)
             filter(lfilter[nph + n * numstages], tmpsmp);
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
             outl[i] += tmpsmp[i] * rolloff;
     }
 
@@ -663,28 +710,28 @@ int SUBnote::noteout(float *outl, float *outr)
     // right channel
     if (stereo)
     {
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
             tmprnd[i] = synth->numRandom() * 2.0f - 1.0f;
         for (int n = 0; n < numharmonics; ++n)
         {
             float rolloff = overtone_rolloff[n];
-            memcpy(tmpsmp, tmprnd, synth->bufferbytes);
+            memcpy(tmpsmp, tmprnd, synth->p_bufferbytes);
             for (int nph = 0; nph < numstages; ++nph)
                 filter(rfilter[nph + n * numstages], tmpsmp);
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
                 outr[i] += tmpsmp[i] * rolloff;
         }
         if (GlobalFilterR != NULL)
             GlobalFilterR->filterout(outr);
     }
     else
-        memcpy(outr, outl, synth->bufferbytes);
+        memcpy(outr, outl, synth->p_bufferbytes);
 
     if (firsttick)
     {
         int n = 10;
-        if (n > synth->buffersize)
-            n = synth->buffersize;
+        if (n > synth->p_buffersize)
+            n = synth->p_buffersize;
         for (int i = 0; i < n; ++i)
         {
             float ampfadein = 0.5f - 0.5f * cosf((float)i / (float)n * PI);
@@ -706,17 +753,17 @@ int SUBnote::noteout(float *outl, float *outr)
     if (aboveAmplitudeThreshold(oldamplitude, newamplitude))
     {
         // Amplitude interpolation
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
         {
             float tmpvol = interpolateAmplitude(oldamplitude, newamplitude, i,
-                                                synth->buffersize);
+                                                synth->p_buffersize);
             outl[i] *= tmpvol * pangainL;
             outr[i] *= tmpvol * pangainR;
         }
     }
     else
     {
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
         {
             outl[i] *= newamplitude * pangainL;
             outr[i] *= newamplitude * pangainR;
@@ -730,8 +777,8 @@ int SUBnote::noteout(float *outl, float *outr)
     {   // Silencer
         if (Legato.msg != LM_FadeIn)
         {
-            memset(outl, 0, synth->bufferbytes);
-            memset(outr, 0, synth->bufferbytes);
+            memset(outl, 0, synth->p_bufferbytes);
+            memset(outr, 0, synth->p_bufferbytes);
         }
     }
     switch (Legato.msg)
@@ -739,7 +786,7 @@ int SUBnote::noteout(float *outl, float *outr)
         case LM_CatchUp : // Continue the catch-up...
             if (Legato.decounter == -10)
                 Legato.decounter = Legato.fade.length;
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
             {   // Yea, could be done without the loop...
                 Legato.decounter--;
                 if (Legato.decounter < 1)
@@ -759,7 +806,7 @@ int SUBnote::noteout(float *outl, float *outr)
             if (Legato.decounter == -10)
                 Legato.decounter = Legato.fade.length;
             Legato.silent = false;
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
             {
                 Legato.decounter--;
                 if (Legato.decounter < 1)
@@ -777,12 +824,12 @@ int SUBnote::noteout(float *outl, float *outr)
         case LM_FadeOut : // Fade-out, then set the catch-up
             if (Legato.decounter == -10)
                 Legato.decounter = Legato.fade.length;
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
             {
                 Legato.decounter--;
                 if (Legato.decounter < 1)
                 {
-                    for (int j = i; j < synth->buffersize; ++j)
+                    for (int j = i; j < synth->p_buffersize; ++j)
                         outl[j] = outr[j] = 0.0f;
                     Legato.decounter = -10;
                     Legato.silent = true;
@@ -812,9 +859,9 @@ int SUBnote::noteout(float *outl, float *outr)
     // Check if the note needs to be computed more
     if (AmpEnvelope->finished() != 0)
     {
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
         {   // fade-out
-            float tmp = 1.0f - (float)i / synth->buffersize_f;
+            float tmp = 1.0f - (float)i / synth->p_buffersize_f;
             outl[i] *= tmp;
             outr[i] *= tmp;
         }
